#
# raccp.rb
#
#    Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#

require 'racc/parser'

$RACCPARSER_DEBUG = false

class Racc


  class RaccParser < Parser

    attr :start
    attr :classname


    def initialize( racc )
      racc.must Racc
      @racc   = racc
      @interf = racc.interf

      @scanner = RaccScanner.new
      @scanner.debug = @yydebug = $RACCPARSER_DEBUG
    end


    def parse( str, fname )
      fname.must String
      @scanner.reset( str )
      @scanner.filename = @filename = fname

      do_parse
    end


    private


    def next_token
      @scanner.scan
    end

    def on_error( tok, val, _values )
      case val
      when Fixnum then v = val.id2name
      when String then v = val
      else
        v = val.inspect
      end
      raise ParseError, <<MES
  parse error:#{@filename}:#{@scanner.lineno}: unexpected token '#{v}'
MES
    end

##### racc generated code begin ###

LR_reduce_table = [
0, 0, :racc_error,
6, 21, :_reduce_1,
0, 22, :_reduce_2,
2, 22, :_reduce_3,
3, 24, :_reduce_4,
1, 24, :_reduce_5,
2, 24, :_reduce_6,
2, 25, :_reduce_7,
3, 25, :_reduce_8,
3, 26, :_reduce_9,
3, 26, :_reduce_10,
1, 28, :_reduce_11,
2, 28, :_reduce_12,
2, 29, :_reduce_13,
2, 29, :_reduce_14,
2, 29, :_reduce_15,
1, 30, :_reduce_16,
2, 30, :_reduce_17,
2, 23, :_reduce_18,
3, 23, :_reduce_19,
5, 31, :_reduce_20,
5, 31, :_reduce_21,
1, 32, :_reduce_22,
1, 32, :_reduce_23,
0, 33, :_reduce_24,
2, 33, :_reduce_25,
1, 27, :_reduce_26,
1, 27, :_reduce_27,
0, 34, :_reduce_28,
2, 34, :_reduce_29,
0, 35, :_reduce_30,
1, 35, :_reduce_31 ]

LR_reduce_n = 32

LR_shift_n = 55

LR_action_table = [
     2,     2,    -1,   -32,     0,     3,    -1,   -32,     3,     4,
    -1,   -32,     0,     5,    -1,   -32,    -1,    -2,    -1,    55,
    10,     7,     6,     9,     4,    11,     7,    12,     9,    13,
    -1,   -32,    12,    14,    11,    17,    13,    18,    -1,   -32,
    -1,    -3,     3,    21,     8,    20,    -1,   -32,    -1,    -5,
     3,    23,    -1,   -32,     3,    26,    -1,   -32,    12,    14,
    11,    17,    13,    18,    -1,   -32,     3,    21,     8,    20,
    -1,   -32,    12,    14,    11,    17,     9,    31,    13,    18,
    -1,   -32,    -1,   -11,     3,    21,     8,    20,    -1,   -32,
     3,    21,     8,    20,    -1,   -32,     8,    34,    -1,   -32,
    -1,   -27,    -1,   -26,     5,    36,     3,    21,     8,    20,
    -1,   -32,    14,    37,    -1,   -32,    15,    40,    16,    39,
    17,    41,    -1,   -32,     5,    42,     3,    23,    -1,   -32,
    -1,    -6,    12,    14,    10,    44,    11,    17,    13,    18,
    -1,   -32,    -1,   -16,     3,    21,     8,    20,    -1,   -14,
    -1,   -12,    -1,   -10,     3,    21,     8,    20,    -1,   -13,
     3,    21,     8,    20,    -1,   -15,    -1,    -7,     8,    46,
    -1,   -32,    -1,    -4,    -1,   -24,    -1,   -18,    -1,   -22,
    -1,   -24,    -1,   -23,    -1,    -1,    15,    40,    16,    39,
    17,    41,    -1,   -32,    -1,    -9,    -1,   -17,    -1,    -8,
     3,    21,     8,    20,    18,    52,    -1,   -28,     3,    21,
     8,    20,    18,    52,    -1,   -28,    -1,   -19,    -1,   -25,
    19,    54,    -1,   -30,     3,    21,     8,    20,    -1,   -32,
    19,    54,    -1,   -30,    -1,   -31,    -1,   -20,    -1,   -29,
    -1,   -21 ]

LR_action_table_ptr = [
     0,     4,     8,    12,    16,    18,    20,    32,    40,    42,
    48,    50,    54,    58,    66,    72,    82,    84,    90,    96,
   100,   102,   104,   112,   116,   124,   130,   132,   142,   144,
   150,   152,   154,   160,   166,   168,   172,   174,   176,   178,
   180,   182,   184,   186,   194,   196,   198,   200,   208,   216,
   218,   220,   224,   230,   234,   236,   238,   240 ]


LR_goto_table = [
    21,     1,    22,     6,    24,     8,    26,    10,    28,    15,
    29,    16,    27,    19,    25,    22,    31,    24,    23,    25,
    28,    27,    29,    16,    27,    28,    30,    29,    29,    30,
    27,    28,    30,    32,    27,    28,    30,    33,    27,    35,
    32,    38,    31,    43,    29,    30,    27,    45,    27,    45,
    27,    45,    33,    47,    33,    48,    32,    49,    27,    50,
    34,    51,    27,    50,    34,    53,    35,    55,    27,    56,
    35,    57 ]

LR_goto_table_ptr = [
     0,    -1,    -1,    -1,     2,    -1,     4,     8,    -1,    12,
    -1,    16,    -1,    20,    24,    28,    -1,    30,    34,    -1,
    -1,    -1,    38,    -1,    40,    42,    -1,    44,    -1,    46,
    -1,    -1,    48,    50,    -1,    -1,    -1,    52,    -1,    -1,
    54,    -1,    -1,    56,    -1,    -1,    -1,    58,    62,    -1,
    -1,    66,    68,    70,    -1,    -1,    -1,    -1 ]


LR_token_table = {
 false => 0,
 Object.new => 1,
 :CLASS => 2,
 :TOKEN => 3,
 :RULE => 4,
 :XEND => 5,
 :XTOKEN => 6,
 :START => 7,
 :STRING => 8,
 :PRECHIGH => 9,
 :PRECLOW => 10,
 :LEFT => 11,
 :RIGHT => 12,
 :NONASSOC => 13,
 ":" => 14,
 "|" => 15,
 :EOL => 16,
 ";" => 17,
 "=" => 18,
 :ACTION => 19 }

DEBUG_PARSER = false

def _reduce_0( val, _values, result )

 result
end

def _reduce_1( val, _values, result )
      @interf.end_register_rule
      @classname = if Integer === val[1] then val[1].id2name else val[1] end
 result
end

def _reduce_2( val, _values, result )

 result
end

def _reduce_3( val, _values, result )

 result
end

def _reduce_4( val, _values, result )
      @interf.end_register_conv
 result
end

def _reduce_5( val, _values, result )

 result
end

def _reduce_6( val, _values, result )
      if @start then
        raise ParseError, "start rule defined twice"
      end
      @start = @interf.get_token( val[1] )
 result
end

def _reduce_7( val, _values, result )
      @interf.register_conv( val[0], val[1] )
 result
end

def _reduce_8( val, _values, result )
      @interf.register_conv( val[1], val[2] )
 result
end

def _reduce_9( val, _values, result )
      @interf.end_register_prec( true )
 result
end

def _reduce_10( val, _values, result )
      @interf.end_register_prec( false )
 result
end

def _reduce_11( val, _values, result )

 result
end

def _reduce_12( val, _values, result )

 result
end

def _reduce_13( val, _values, result )
      @interf.register_prec( :Left, val[1] )
 result
end

def _reduce_14( val, _values, result )
      @interf.register_prec( :Right, val[1] )
 result
end

def _reduce_15( val, _values, result )
      @interf.register_prec( :Nonassoc, val[1] )
 result
end

def _reduce_16( val, _values, result )
      result = val
 result
end

def _reduce_17( val, _values, result )
      result.push val[1]
 result
end

def _reduce_18( val, _values, result )

 result
end

def _reduce_19( val, _values, result )

 result
end

def _reduce_20( val, _values, result )
      @interf.register_rule @interf.get_token(val[0]), val[2], val[3], val[4]
 result
end

def _reduce_21( val, _values, result )
      @interf.register_rule( nil, val[2], val[3], val[4] )
 result
end

def _reduce_22( val, _values, result )

 result
end

def _reduce_23( val, _values, result )

 result
end

def _reduce_24( val, _values, result )
      result = []
 result
end

def _reduce_25( val, _values, result )
      result.push val[1]
 result
end

def _reduce_26( val, _values, result )
      result = @interf.get_token( val[0] )
 result
end

def _reduce_27( val, _values, result )
      result = @interf.get_token( eval '"' + val[0] + '"' )
 result
end

def _reduce_28( val, _values, result )

 result
end

def _reduce_29( val, _values, result )
      result = val[1]
 result
end

def _reduce_30( val, _values, result )
      result = ''
 result
end

def _reduce_31( val, _values, result )

 result
end
##### racc generated code end #####
  end

end
